/* Name: bootloaderconfig.h
 * Project: USBaspLoader
 * Author: Christian Starkjohann
 * Creation Date: 2007-12-08
 * Tabsize: 4
 * Copyright: (c) 2007 by OBJECTIVE DEVELOPMENT Software GmbH
 * License: GNU GPL v2 (see License.txt)
 * This Revision: $Id: bootloaderconfig.h 729 2009-03-20 09:03:58Z cs $
 */

#ifndef __bootloaderconfig_h_included__
#define __bootloaderconfig_h_included__

/*
General Description:
This file (together with some settings in Makefile) configures the boot loader
according to the hardware.

This file contains (besides the hardware configuration normally found in
usbconfig.h) two functions or macros: bootLoaderInit() and
bootLoaderCondition(). Whether you implement them as macros or as static
inline functions is up to you, decide based on code size and convenience.

bootLoaderInit() is called as one of the first actions after reset. It should
be a minimum initialization of the hardware so that the boot loader condition
can be read. This will usually consist of activating a pull-up resistor for an
external jumper which selects boot loader mode.

bootLoaderCondition() is called immediately after initialization and in each
main loop iteration. If it returns TRUE, the boot loader will be active. If it
returns FALSE, the boot loader jumps to address 0 (the loaded application)
immediately.

*/

/* ------------------------------------------------------------------------- */
/* ---------------------- feature / code size options ---------------------- */
/* ------------------------------------------------------------------------- */

#define HAVE_EEPROM_PAGED_ACCESS    0
/* If HAVE_EEPROM_PAGED_ACCESS is defined to 1, page mode access to EEPROM is
 * compiled in. Whether page mode or byte mode access is used by AVRDUDE
 * depends on the target device. Page mode is only used if the device supports
 * it, e.g. for the ATMega88, 168 etc. You can save quite a bit of memory by
 * disabling page mode EEPROM access. Costs ~ 138 bytes.
 */
#define HAVE_EEPROM_BYTE_ACCESS     1
/* If HAVE_EEPROM_BYTE_ACCESS is defined to 1, byte mode access to EEPROM is
 * compiled in. Byte mode is only used if the device (as identified by its
 * signature) does not support page mode for EEPROM. It is required for
 * accessing the EEPROM on the ATMega8. Costs ~54 bytes.
 */
#define BOOTLOADER_CAN_EXIT         1
/* If this macro is defined to 1, the boot loader will exit shortly after the
 * programmer closes the connection to the device. Costs ~36 bytes.
 */
#define HAVE_CHIP_ERASE             1
/* If this macro is defined to 1, the boot loader implements the Chip Erase
 * ISP command. Otherwise pages are erased on demand before they are written.
 */
#define HAVE_FLASH_BYTE_ACCESS      1
#define HAVE_FUSE_ACCESS            1
#define HAVE_SIGNATURE_ACCESS       1

#define BOOTLOADER_EMU		    0
#define USE_DFU_CALL                0

/* ------------------------------------------------------------------------- */

/* Example configuration: Port D bit 3 is connected to a jumper which ties
 * this pin to GND if the boot loader is requested. Initialization allows
 * several clock cycles for the input voltage to stabilize before
 * bootLoaderCondition() samples the value.
 * We use a function for bootLoaderInit() for convenience and a macro for
 * bootLoaderCondition() for efficiency.
 */

#ifndef __ASSEMBLER__   /* assembler cannot parse function definitions */

#ifndef MCUCSR          /* compatibility between ATMega8 and ATMega88 */
#   define MCUCSR   MCUSR
#endif
#define HWB_BIT     PD7

#define CHK_EXTRF		1
#define USE_HWB			1
#define CHK_HWB_LEVEL		0	/* usefull with the slide switch */

static inline void  bootLoaderInit(void)
{
#if CHK_EXTRF && !USE_HWB
    PORTD |= (1<<HWB_BIT);     /* activate pull-up */
    /* If this was not an external reset/usb reset, ignore */
    if((MCUCSR & ((1<<PORF)|(1<<BORF)|(1<<WDRF))) && !(MCUCSR & (1<<EXTRF))
           || (PIND & (1<<HWB_BIT)))
        leaveBootloader();
#elif CHK_EXTRF
    /* If this was not an external reset/usb reset, ignore */
     if((MCUCSR & ((1<<PORF)|(1<<BORF)|(1<<WDRF))) && !(MCUCSR & (1<<EXTRF)))
        leaveBootloader();
#elif !USE_HWB
    PORTD |= (1<<HWB_BIT);     /* activate pull-up */
    if((PIND & (1<<HWB_BIT)))    /* If this was not an external reset, ignore */
        leaveBootloader();
#endif
    MCUCSR = 0;                     /* clear all reset flags for next time */
}

static inline void  bootLoaderEnter(void)
{
    DDRD  |= (1<< PD7); /* connect_led */
    DDRD  |= (1<< PD6); /* busy_led */
}

static inline void  bootLoaderExit(void)
{
    DDRD  &= ~(1<< PD7);
    PORTD &= ~(1 << PD7);     /* undo bootLoaderInit() changes */
    DDRD  &= ~(1<< PD6);
    PORTD &= ~(1<< PD6);
}

static inline void connect_led_on() {
    PORTD |= (1<< PD7);
}

static inline void connect_led_off() {
    PORTD &= ~(1<< PD7);
}

static inline void busy_led_on() {
    PORTD |= (1<< PD6);
}

static inline void busy_led_off() {
    PORTD &= ~(1<< PD6);
}

#define bootLoaderEnterCondition()   (1)
#if CHK_HWB_LEVEL
#define bootLoaderExitCondition()   (PIND & (1<< HWB_BIT))
#else
#define bootLoaderExitCondition()   (0)
#endif

#endif /* __ASSEMBLER__ */

/* ------------------------------------------------------------------------- */

#endif /* __bootloader_h_included__ */
